<?php

/*------------------------------------------------------------------------
# com_invoices - Invoices for Joomla
# ------------------------------------------------------------------------
# author				Germinal Camps
# copyright 			Copyright (C) 2012 JoomlaInvoices.com. All Rights Reserved.
# @license				http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
# Websites: 			http://www.JoomlaInvoices.com
# Technical Support:	Forum - http://www.JoomlaFinances.com/forum
-------------------------------------------------------------------------*/

/** ensure this file is being included by a parent file */
defined('_JEXEC') or die('Restricted access');

jimport('joomla.plugin.plugin');

class PlgInvoicesPayment_paypal extends JPlugin {

	var $_payment_type = 'payment_paypal';

	function __construct(& $subject, $config) {
		parent::__construct($subject, $config);
		$this->loadLanguage( '', JPATH_ADMINISTRATOR );
	}

	function _process()
	{
		$data = JRequest::get('post');
		$errors = array();

		// $this->_logIPNResponse($data);
		if ($ipnError = $this->_validateIPN($data)) {
			$errors[] = $ipnError;
		}

		// prepare some data
		$item_number_array	= explode(':', @$data['item_number']);
		$data["payment_id"] = (int) @$item_number_array[0];

		$payment_details = $this->_getFormattedPaymentDetails($data);

		// process the payment based on its type
		if ( ! empty($data['txn_type']) && $data['txn_type'] != 'subscr_cancel') {
			if ( ! ($user =& $this->_getUser( (int)$data['custom'], $data['payer_email'], $data['payer_id'] ))) {
				$errors[] = JText::_('Paypal Message Unknown User');

				$user = JFactory::getUser();
				$user->set('id', 0);
			}

			if ($data['txn_type'] == 'web_accept') {
				$errors = $this->_processSale($data, $user, $payment_details, $errors);
			}

		}

		return count($errors) ? implode("\n", $errors) : 'processed';
	}

	function _processSale($data, $user, $payment_details, $errors)
	{
		/*
		* validate the payment data
		*/
		// $this->_logIPNResponse('_processSale');
		// $this->_logIPNResponse($data);
		if (@$data['receiver_email'] != $this->_getParam('receiver_email') && @$data['business'] != $this->_getParam('receiver_email')) {
			$errors[] = JText::_('PAYPAL MESSAGE RECEIVER INVALID');
		}

		//germi
		//we load the payment
		$payment = InvoicesHelper::getPaymentData($data['payment_id']);

		// here we will make sure we haven't received less money,
		// we can't use the equal operation because of probable tax additions to the final sum
		if ((float)$data['mc_gross'] < (float)$payment->payment_amount) {
			$errors[] = JText::_('PAYPAL MESSAGE AMOUNT INVALID');
		}

		if (empty($data['payment_status']) || ($data['payment_status'] != 'Completed' && $data['payment_status'] != 'Pending')) {
			$errors[] = JText::sprintf('PAYPAL MESSAGE STATUS INVALID', @$data['payment_status']);
		}

		if(count($errors)) {

			$payment->payment_status = 0; //unpaid - errors
		}
		elseif(@$data['payment_status'] == 'Pending'){
			$payment->payment_status = 2; //pending
		}
		else {
			$payment->payment_status = 1; //paid
		}

		$payment->payment_type = $this->_payment_type;
		$payment->payment_details = $payment_details;
		$payment->payment_datetime = date("Y-m-d H:i:s");

		if(!count($errors)) {
			$this->update_payment($payment);
		}

		return $errors;
	}

	//germi
	function update_payment($payment){

		$model = $this->getPaymentsModel();
		$model->store(get_object_vars($payment));

	}

	private function getPaymentsModel()
	{
		if (!class_exists( 'PaymentsModelPayment' ))
		{
			// Build the path to the model based upon a supplied base path
			$path = JPATH_SITE.DS.'administrator'.DS.'components'.DS.'com_invoices'.DS.'models'.DS.'payment.php';
			$false = false;

			// If the model file exists include it and try to instantiate the object
			if (file_exists( $path )) {
				require_once( $path );
				if (!class_exists( 'PaymentsModelPayment' )) {
					JError::raiseWarning( 0, 'Model class PaymentsModelPayment not found in file.' );
					return $false;
				}
			} else {
				JError::raiseWarning( 0, 'Model PaymentsModelPayment not supported. File not found.' );
				return $false;
			}
		}

		$model = new PaymentsModelPayment();
		return $model;
	}

	function _logIPNResponse($data)
	{
		$f = fopen(JPATH_CACHE . '/ipn_paypal.txt', 'a');
		fwrite($f, "\n" . date('F j, Y, g:i a') . "\n");
		fwrite($f, print_r($data, true));
		fclose($f);
	}

	function _validateIPN($data)
	{

		$paypal_sandbox_url = $this->_getPaypalIPNUrl();

		//https://developer.paypal.com/docs/classic/ipn/ht_ipn/
		// read the IPN message sent from PayPal and prepend 'cmd=_notify-validate'
		$req = 'cmd=_notify-validate';
		foreach ($data as $key => $value) {
			if ($key != 'view' && $key != 'layout') {
				$value = urlencode($value);
				$req .= "&$key=$value";
			}
		}
		// $this->_logIPNResponse($req);
		// Step 2: POST IPN data back to PayPal to validate
		$ch = curl_init($paypal_sandbox_url);
		curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $req);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
		curl_setopt($ch, CURLOPT_FORBID_REUSE, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Connection: Close'));
		curl_setopt($ch, CURLOPT_SSLVERSION, 6); // CURL_SSLVERSION_TLSv1_2

		// In wamp-like environments that do not come bundled with root authority certificates,
		// please download 'cacert.pem' from "https://curl.haxx.se/docs/caextract.html" and set
		// the directory path of the certificate as shown below:
		// curl_setopt($ch, CURLOPT_CAINFO, dirname(__FILE__) . '/cacert.pem');
		if ( !($res = curl_exec($ch)) ) {
			// $this->_logIPNResponse($res);
			// $this->_logIPNResponse(curl_error($ch));
		  curl_close($ch);
		  return "Got " . curl_error($ch) . " when processing IPN data";
		}
		curl_close($ch);
		// $this->_logIPNResponse($res);
		if (strcmp ($res, "VERIFIED") == 0) {
		  // The IPN is verified, process it
			return '';
		} else if (strcmp ($res, "INVALID") == 0) {
		  // IPN invalid, log for manual investigation
			return JText::_('PAYPAL ERROR IPN VALIDATION');
		}

		return JText::_('PAYPAL ERROR IPN VALIDATION');

	}

	function onRenderPaymentOptions( $row, $user )
	{
		/*
		* get all necessary data and prepare vars for assigning to the template
		*/

		$compparams = JComponentHelper::getParams( 'com_invoices' );

		$vars = new JObject();

		$db = JFactory::getDBO();

		if(!$row->currency_id) $row->currency_id = $compparams->get('currency_id', 1);
		$query = "SELECT * FROM #__invoices_currencies
		WHERE id = ".(int)$row->currency_id;
		$db->setQuery($query);
		$currency = $db->loadObject();

		$vars->receiver_email = $this->_getParam( 'receiver_email' );
		$vars->return_url = JURI::root()."index.php?option=com_invoices&task=process_payment&ptype={$this->_payment_type}&paction=display_message";
		$vars->cancel_url = JURI::root()."index.php?option=com_invoices&task=process_payment&ptype={$this->_payment_type}&paction=cancel&item_number=".$row->id;
		$vars->notify_url = JURI::root()."index.php?option=com_invoices&task=process_payment&ptype={$this->_payment_type}&paction=process&tmpl=component&item_number=".$row->id;
		$vars->currency = $currency->currency_code;
		$vars->custom = $user->get('id');
		$vars->note = $this->params->get( 'description' );
		$vars->subParams =& $subParams;
		$vars->action_url = $this->_getPaypalUrl();
		$vars->row =& $row;
		$vars->user =& $user;

		$html = $this->_getLayout('form', $vars);

		$text = array();
		$text[] = $html;
		$text[] = $this->params->get( 'title', 'PayPal' );

		return $text;
	}

	function onProcessPayment( $row, $user )
	{
		$ptype 		= JRequest::getVar( 'ptype' );
		if ($ptype == $this->_payment_type)
		{
			$paction 	= JRequest::getVar( 'paction' );
			$html = "";

			switch ($paction) {
				case "display_message":

				break;
				case "process":
				$html .= $this->_process();

				echo $html;
				$app = JFactory::getApplication();
				//$app->close();
				break;
				case "cancel":

				break;
				default:

				break;
			}

			return $html;
		}

	}

	function & _getUser($id, $email, $unique_gateway_id)
	{
		$user = JFactory::getUser($id);

		return $user;
	}

	function _getPaypalUrl($full = true)
	{
		$url = $this->params->get('sandbox') ? 'www.sandbox.paypal.com' : 'www.paypal.com';

		if ($full) {
			$url = 'https://' . $url . '/cgi-bin/webscr';
		}

		return $url;
	}

	function _getPaypalIPNUrl()
	{
		$sandbox = $this->params->get('sandbox') ? "sandbox." : "";

		$url = 'https://ipnpb.'.$sandbox.'paypal.com/cgi-bin/webscr';

		return $url;
	}

	function _getParam($name, $default = '')
	{
		$sandbox_param = "sandbox_$name";
		$sb_value = $this->params->get($sandbox_param);

		if ($this->params->get('sandbox') && !empty($sb_value)) {
			$param = $this->params->get($sandbox_param, $default);
		}
		else {
			$param = $this->params->get($name, $default);
		}

		return $param;
	}

	function _getLayout($layout, $vars = false, $plugin = '', $group = 'invoices')
	{
		if ( ! $plugin) {
			$plugin = $this->_payment_type;
		}

		ob_start();
		$layout = $this->_getLayoutPath( $plugin, $group, $layout );
		include($layout);
		$html = ob_get_contents();
		ob_end_clean();

		return $html;
	}

	function _getLayoutPath($plugin, $group, $layout = 'default')
	{
		$app = JFactory::getApplication();

		// get the template and default paths for the layout
		$templatePath = JPATH_BASE.DS.'templates'.DS.$app->getTemplate().DS.'html'.DS.'plugins'.DS.$group.DS.$plugin.DS.$plugin.DS.$layout.'.php';
		$defaultPath = JPATH_BASE.DS.'plugins'.DS.$group.DS.$plugin.DS.$plugin.DS.'tmpl'.DS.$layout.'.php';

		// if the site template has a layout override, use it
		jimport('joomla.filesystem.file');
		if (JFile::exists( $templatePath ))
		{
			return $templatePath;
		}
		else
		{
			return $defaultPath;
		}
	}

	function _getFormattedPaymentDetails($data)
	{
		$separator = "\n";
		$formatted = array();

		foreach ($data as $key => $value) {
			if ($key != 'view' && $key != 'layout' && $key != 'custom') {
				$formatted[] = $key . ' = ' . $value;
			}
		}

		return count($formatted) ? implode("\n", $formatted) : '';
	}

}

if ( ! function_exists('plg_invoices_escape')) {

	function plg_invoices_escape($var)
	{
		return htmlspecialchars($var, ENT_COMPAT, 'UTF-8');
	}
}
